// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <math.h>
// GL a GLU kniznica
//#include <GL/gl.h>
//#include <GL/glu.h>

// .h subor
#include "m3Dmath.h"

// vypocet velkosti vektora ************************************************
float m3Dvector::Magnitude(void)
{
 return(sqrt(x*x+y*y+z*z));
}

// normalizacia vektora ****************************************************
void m3Dvector::Normalize(void)
{
 M=Magnitude();

 if (M!=0.0f) { x/=M; y/=M; z/=M; }
}

// vzdialenost dvoch bodov *************************************************
float m3Dmath::Distance(m3Dvector pA,m3Dvector pB)
{
 return(sqrt((pB.x-pA.x)*(pB.x-pA.x)+(pB.y-pA.y)*(pB.y-pA.y)+(pB.z-pA.z)*(pB.z-pA.z)));
}

// skalarny sucin dvoch vektorov *******************************************
float m3Dmath::Dot(m3Dvector vector1,m3Dvector vector2)
{
 return((vector1.x*vector2.x)+(vector1.y*vector2.y)+(vector1.z*vector2.z));
}

// vektorovy sucin dvoch vektorov ******************************************
void m3Dmath::Cross(m3Dvector vector1,m3Dvector vector2,m3Dvector *vector3)
{
 vector3->x=vector1.y*vector2.z-vector1.z*vector2.y;
 vector3->y=vector1.z*vector2.x-vector1.x*vector2.z;
 vector3->z=vector1.x*vector2.y-vector1.y*vector2.x;
}

// vypocet normaly *********************************************************
void m3Dmath::CalcNormal(float p1,float p2,float p3,
                         float q1,float q2,float q3,
                         float r1,float r2,float r3,
                         float *a,float *b,float *c)
{
 v11=q1-p1; v12=q2-p2; v13=q3-p3; // vypocet prvkov 2 vektorov
 v21=r1-p1; v22=r2-p2; v23=r3-p3;

 n1=v12*v23-v13*v22;              // vektorovy sucin 2 vektorov
 n2=v13*v21-v11*v23;
 n3=v11*v22-v12*v21;

 // normalizacia vektora
 NormaliseNormal(n1,n2,n3,&nn1,&nn2,&nn3);
 // vysledok
 *a=nn1; *b=nn2; *c=nn3;
}

// normalizacia normaly ****************************************************
void m3Dmath::NormaliseNormal(float n1,float n2,float n3,
                              float *a,float *b,float *c)
{
 // ziskanie velkosti vektora
 d=sqrt(n1*n1+n2*n2+n3*n3);
 // uprava prvkov vektora, vysledok
 if (d!=0.0f) { *a=n1/d; *b=n2/d; *c=n3/d; }
 else { *a=0.0f; *b=0.0f; *c=0.0f; }
}

// urcenie polohy bodu voci rovine *****************************************
PointPos m3Dmath::ClassifyPoint(m3Dvector normal,float d,m3Dvector point)
{
 dist1=normal.x*point.x+normal.y*point.y+normal.z*point.z+d;

 if (dist1>0.0f) return POINTPOS_FRONT;
 else if (dist1<0.0f) return POINTPOS_BEHIND;
	  else return POINTPOS_ON;
}

// vypocet priesecnika hrany trojuholnika a roviny tvorenej trojuholnikom **
int m3Dmath::EdgePlaneIntersection(m3Dvector p1,m3Dvector p2,m3Dvector plane_normal,float d,m3Dvector *IP)
{
 side1=ClassifyPoint(plane_normal,d,p1);
 side2=ClassifyPoint(plane_normal,d,p2);

 if ((side1==side2)||(side1==POINTPOS_ON)||(side2==POINTPOS_ON)) return(0);
 
 // vypocet prvkov vektora P1->P2
 vec1=p2-p1;
 vec1.Normalize();
 // ziskanie vzdialenosti bodu P1 od roviny
 Numerator=-(plane_normal.x*p1.x+
             plane_normal.y*p1.y+
             plane_normal.z*p1.z+
             d);
 // skalarny sucin (dot product) dvoch vektorov
 Denominator=Dot(plane_normal,vec1);

 // v pripade, ze vektor P1->P2 je kolmy na normalu trojuholnika, odovzda sa bod P1
 if (Denominator==0.0f) *IP=p1;

 D=Numerator/Denominator;

 // priesecnik vektora P1->P2 a roviny tvorenej trojuholnikom
 IP->x=(p1.x+(vec1.x*D));
 IP->y=(p1.y+(vec1.y*D));
 IP->z=(p1.z+(vec1.z*D));

 return(1);
}

// vypocet novych texturovacich suradnic  v priesecnikoch ******************
void m3Dmath::ComputeTextureCoords(m3Dvector A,m3Dvector B,m3Dvector C,
                                   m3Dvector At,m3Dvector Bt,m3Dvector Ct,
                                   unsigned int a1,
                                   unsigned int a2,
                                   m3Dvector p1,m3Dvector p2,
                                   m3Dvector *p1t,m3Dvector *p2t)
{
 switch (a1) {
  case 1: // p1t pretina hranu A->B
   dist1=Distance(A,B); dist2=Distance(A,p1);
   vec1=Bt-At;

   Pf=dist2/dist1;
   p1t->x=At.x+vec1.x*Pf; p1t->y=At.y+vec1.y*Pf;
   break;
  case 2: // p1t pretina hranu B->C
   dist1=Distance(B,C); dist2=Distance(B,p1);
   vec1=Ct-Bt;

   Pf=dist2/dist1;
   p1t->x=Bt.x+vec1.x*Pf; p1t->y=Bt.y+vec1.y*Pf;
   break;
  case 3: // p1t pretina hranu C->A
   dist1=Distance(C,A); dist2=Distance(C,p1);
   vec1=At-Ct;

   Pf=dist2/dist1;
   p1t->x=Ct.x+vec1.x*Pf; p1t->y=Ct.y+vec1.y*Pf;
   break;
 }

 switch (a2) {
  case 1: // p2t pretina hranu A->B
   dist1=Distance(A,B); dist2=Distance(A,p2);
   vec1=Bt-At;

   Pf=dist2/dist1;
   p2t->x=At.x+vec1.x*Pf; p2t->y=At.y+vec1.y*Pf;
   break;
  case 2: // p2t pretina hranu B->C
   dist1=Distance(B,C); dist2=Distance(B,p2);
   vec1=Ct-Bt;

   Pf=dist2/dist1;
   p2t->x=Bt.x+vec1.x*Pf; p2t->y=Bt.y+vec1.y*Pf;
   break;
  case 3: // p2t pretina hranu C->A
   dist1=Distance(C,A); dist2=Distance(C,p2);
   vec1=At-Ct;

   Pf=dist2/dist1;
   p2t->x=Ct.x+vec1.x*Pf; p2t->y=Ct.y+vec1.y*Pf;
   break;
 }
 
 // INFO: povodne a nove texturovacie suradnice
 //fprintf(stderr,"%f %f %f %f %f %f\n",At.x,At.y,Bt.x,Bt.y,Ct.x,Ct.y);
 //fprintf(stderr,"%f %f %f %f\n",p1t->x,p1t->y,p2t->x,p2t->y);
}
