#ifndef _M3DMATH_H_
#define _M3DMATH_H_

enum PointPos // poloha bodu
{
 POINTPOS_FRONT=1,   // bod lezi pred rovinou
 POINTPOS_BEHIND=-1, // bod lezi za rovinou
 POINTPOS_ON=0       // bod lezi v rovine
};

enum TriPos // poloha trojuholnika
{
 TRIPOS_FRONT,   // trojuholnik lezi cely pred rovinou
 TRIPOS_BEHIND,  // trojuholnik lezi cely za rovinou
 TRIPOS_ON,      // trojuholnik lezi v rovine (je s nou rovnobezny)
 TRIPOS_SPANNING // trojuholnik lezi na oboch stranach roviny (pretina rovinu)
};

// trieda pre pracu s 3D bodmi a vektormi
class m3Dvector {
public:
 // konstruktor
 m3Dvector() {}
 // konstruktor s inicializaciou dat
 m3Dvector(float a,float b,float c) { x=a; y=b; z=c; }

 // pretazeny operator +
 m3Dvector operator+(m3Dvector mv) { return m3Dvector(mv.x+x,mv.y+y,mv.z+z); }
 // pretazeny operator -
 m3Dvector operator-(m3Dvector mv) { return m3Dvector(x-mv.x,y-mv.y,z-mv.z); }
 // pretazeny operator *
 m3Dvector operator*(float num) { return m3Dvector(x*num,y*num,z*num); }
 // prvky vektora (poloha 3D bodu)
 float x,y,z;

 // velkost vektora
 float M;
 // vypocet velkosti vektora
 float Magnitude(void);
 // normalizacia vektora
 void Normalize(void);
};

// trieda matematicko-geometrickych vypoctov
class m3Dmath {
public:
 // konstruktor
 m3Dmath() {}

 // vzdialenost dvoch bodov
 float Distance(m3Dvector pA,m3Dvector pB);
	 
 // skalarny sucin dvoch vektorov
 float Dot(m3Dvector vector1,m3Dvector vector2);

 // vektorovy sucin dvoch vektorov
 void Cross(m3Dvector vector1,m3Dvector vector2,m3Dvector *vector3);

 // vypocet normaly
 void CalcNormal(float p1,float p2,float p3,
                 float q1,float q2,float q3,
                 float r1,float r2,float r3,
                 float *a,float *b,float *c);

 // normalizacia normaly
 void NormaliseNormal(float n1,float n2,float n3,
                      float *a,float *b,float *c);

 // urcenie polohy bodu voci rovine	 
 PointPos ClassifyPoint(m3Dvector normal,float d,m3Dvector point);

 // vypocet priesecnika hrany trojuholnika a roviny tvorenej trojuholnikom
 int EdgePlaneIntersection(m3Dvector p1,m3Dvector p2,m3Dvector plane_normal,float d,m3Dvector *IP);

 // vypocet novych texturovacich suradnic v priesecnikoch
 void ComputeTextureCoords(m3Dvector A,m3Dvector B,m3Dvector C,
	                       m3Dvector At,m3Dvector Bt,m3Dvector Ct,
                           unsigned int a1,
                           unsigned int a2,
                           m3Dvector p1,m3Dvector p2,
                           m3Dvector *p1t,m3Dvector *p2t);
private:
 // nasledovne premenne su pouzivane pri vypoctoch normal
 float v11,v12,v13;          // 1. vektor
 float v21,v22,v23;          // 2. vektor
 float n1,n2,n3,nn1,nn2,nn3; // pomocne premenne
 float d;                    // velkost vektora

 PointPos side1,side2; // urcenie polohy bodu

 float dist1,dist2; // vypocet vzdialenosti

 m3Dvector vec1,vec2; // pomocne vektory

 float Numerator,Denominator,D; // pre vypocet priesecnika bodu s rovinou

 float Pf; // vypocet pomeru vzdialenosti
};
 
#endif
