// hlavickove subory *******************************************************
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

// .h subor
#include "mTexture.h"

// SDL nacitanie suborov s texturami
SDL_Surface *TextureImage;

// konstruktor *************************************************************
mTexture::mTexture()
{
 texture_id[0]=0;
 texture_count=0;

 crosshair_tex_id=0;
	
 TextureImage=NULL;
}

// destruktor **************************************************************
mTexture::~mTexture()
{
 for (t_a=0;t_a<texture_count;t_a++) glDeleteTextures(1,&texture_id[t_a]);
 glDeleteTextures(1,&crosshair_tex_id);
}

// nahratie textury ********************************************************
int mTexture::LoadTexture(GLuint id,char *myfile,GLuint mode)
{
 TextureImage=IMG_Load(myfile);
	
 // INFO: identifikator a nazov suboru textury
 //fprintf(stderr,"textura: %i %s\n",id,myfile);
	
 // nahratie suboru s bitmapou
 if (TextureImage!=NULL) {
  glGenTextures(1,&texture_id[id]);
  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,texture_id[id]);

  // nastavenie filteringu	 
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  if (mode&TEXTURE_TRILINEAR) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
  } else {
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
	}

  // nastavenie opakovania sa textury
  if (mode&TEXTURE_CLAMP) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  } else {
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
    }
	
  // vygenerovanie textury
  if (mode&TEXTURE_MIPMAP) {
   // vygenerovanie MIPMAP textury
   gluBuild2DMipmaps(GL_TEXTURE_2D,4,TextureImage->w,TextureImage->h,
                     GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
  } else {
     // vygenerovanie standardnej 2D textury
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,TextureImage->w,TextureImage->h,
                  0,GL_RGB,GL_UNSIGNED_BYTE,TextureImage->pixels);
    }

  texture_count++;
 }
 else return(1);
 
 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;	 

 return(0);
}

// zrusenie OpenGL textury *************************************************
void mTexture::FreeTexture(GLuint id)
{
 glDeleteTextures(1,&id);
}

// nahratie textury zameriavacieho kriza ***********************************
int mTexture::LoadCrosshairTexture(char *myfile)
{
 // nahratie suboru s bitmapou
 if ((TextureImage=IMG_Load(myfile))) {
  glGenTextures(1,&crosshair_tex_id);
  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,crosshair_tex_id);
  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  // nastavenie opakovania sa textury
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  // vygenerovanie textury
  glTexImage2D(GL_TEXTURE_2D,0,3,TextureImage->w,TextureImage->h,
               0,GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// vykreslenie zameriavacieho kriza ****************************************
int mTexture::DrawCrosshair(GLfloat w,GLfloat h)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT|GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky
 glEnable(GL_TEXTURE_2D);  // zapnutie texturovania

 glBlendFunc(GL_ONE,GL_ONE); // zapnutie priesvitnosti
 glEnable(GL_BLEND);

 glMatrixMode(GL_PROJECTION); // uprava projekcnej matice
 glPushMatrix();
 glLoadIdentity();
 glOrtho(0.0f,w,0.0f,h,-1.0f,1.0f); // pravouhly suradnicovy system
 glMatrixMode(GL_MODELVIEW);  // uprava pohladovej matice
 glPushMatrix();
 glLoadIdentity();

 // pripojenie textury zameriavacieho kriza
 glBindTexture(GL_TEXTURE_2D,crosshair_tex_id);
 // zameriavaci kriz na stred okna
 glTranslatef(w/2.0f,h/2.0f,0.0f);
 // stvoruholnik
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glVertex3f(-32.0f,-32.0f,0.0f);
  glTexCoord2f(1.0f,0.0f); glVertex3f( 32.0f,-32.0f,0.0f);
  glTexCoord2f(1.0f,1.0f); glVertex3f( 32.0f, 32.0f,0.0f);
  glTexCoord2f(0.0f,1.0f); glVertex3f(-32.0f, 32.0f,0.0f);
 glEnd();

 glPopMatrix(); // pohladova
 glMatrixMode(GL_PROJECTION);
 glPopMatrix(); // projekcna
 glMatrixMode(GL_MODELVIEW); // aktualna je vzdy pohladova

 glDisable(GL_BLEND); // vypnutie priesvitnosti

 glPopAttrib(); // vyber ulozenych atributov

 return(0);
}
