#ifndef _MTRIANGLE_H_
#define _MTRIANGLE_H_

typedef struct __TRIANGLE {
 GLuint id; // identifikator trojuholnika

 GLuint tex_id; // identifikator textury trojuholnika

 GLfloat x1,y1,z1; // suradnice 3 vrcholov
 GLfloat x2,y2,z2;
 GLfloat x3,y3,z3;

 GLfloat tr_nx,tr_ny,tr_nz; // normala trojuholnika
 GLfloat D;                 // D - prvok rovnice roviny
                            // potrebny pre rozne dalsie vypocty
 GLfloat xn1,yn1,zn1; // normaly pre kazdy z 3 vrcholov
 GLfloat xn2,yn2,zn2;
 GLfloat xn3,yn3,zn3;

 GLfloat xt1,yt1; // koordinaty textur pre kazdy z 3 vrcholov
 GLfloat xt2,yt2;
 GLfloat xt3,yt3;

 GLfloat tr_cn1,tr_cn2, tr_cn3, tr_cn4;  // normala +d 1. strany pre vypocet kolizie
 GLfloat tr_cn5,tr_cn6, tr_cn7, tr_cn8;  // normala +d 2. strany pre vypocet kolizie
 GLfloat tr_cn9,tr_cn10,tr_cn11,tr_cn12; // normala +d 3. strany pre vypocet kolizie

 GLfloat color[4]; // farba trojuholnika (pre zobrazenie SLNBSP stromu)
 
 GLuint collision_tested; // trojuholnik je/nie je testovany pocas detekcie kolizii
 
 __TRIANGLE *next; // odkaz na dalsi trojuholnik v zozname
} _TRIANGLE;

// struktura predstavujuca uzol SLNBSP stromu
typedef struct __SolidLeafNode // Solid Leaf uzol
{
 _TRIANGLE *splitter;     // deliaca rovina
 __SolidLeafNode *front;  // odkaz na "predneho" potomka
 __SolidLeafNode *behind; // odkaz na "zadneho" potomka
 GLubyte leaf;            // uzol je typu "leaf" - list stromu
 GLubyte solid;           // uzol je typu "solid" - deli priestor na binarne casti
} _SolidLeafNode;

// trieda premennych a funkcii pre pracu s trojuholnikom
class mTriangle {
public:
 mTriangle();
 virtual ~mTriangle();

 // pridanie trojuholnika do zoznamu
 int triangle_add(GLuint tid,
                  GLfloat a1,GLfloat a2,GLfloat a3,
                  GLfloat b1,GLfloat b2,GLfloat b3,
                  GLfloat c1,GLfloat c2,GLfloat c3,
                  GLfloat d1,GLfloat d2,
                  GLfloat e1,GLfloat e2,
                  GLfloat f1,GLfloat f2,
                  GLuint ctested);
 // vlozenie noveho trojuholnika do zoznamu pocas tvorby SLNBSP stromu
 int triangle_insert(_TRIANGLE *newtriangle,
                     GLuint id,
                     GLuint tid,
                     GLfloat a1,GLfloat a2,GLfloat a3,
                     GLfloat b1,GLfloat b2,GLfloat b3,
                     GLfloat c1,GLfloat c2,GLfloat c3,
                     GLfloat d1,GLfloat d2,
                     GLfloat e1,GLfloat e2,
                     GLfloat f1,GLfloat f2,
                     GLuint ctested);

 // vytvorenie OpenGL textur pre trojuholniky
 int texture_add(GLuint id,char *texname);

 // ziskanie odkazu na zoznam trojuholnikov
 _TRIANGLE *GetFirstTriangle(void);

 // vykreslenie vsetkych trojuholnikov
 void DrawTriangles(void);
 // vykreslenie normal vsetkych trojuholnikov
 void DrawTrianglesNormals(GLfloat size);

 GLuint map_tris_count;  // pocet trojuholnikov nacitanych z mapy sceny
 GLuint all_tris_count;  // celkovy pocet trojuholnikov v zozname
 GLuint all_nodes_count; // celkovy pocet uzlov SLNBSP stromu
 GLuint tested_nodes;    // pocet testovanych uzlov
 
 // vytvorenie Solid Leaf Node BSP stromu
 int create_SLNBSPT(void);
 // vykreslenie Solid Leaf Node BSP stromu
 int draw_SLNBSPT(GLfloat cam_pos_x,
                  GLfloat cam_pos_y,
				  GLfloat cam_pos_z,
				  GLubyte dt);
 // zrusenie Solid Leaf Node BSP stromu
 int destroy_SLNBSPT(void);
private:
 _TRIANGLE *m_first_triangle,*m_last_triangle; // prvy, posledny trojuholnik
 _TRIANGLE *m_t;                               // pomocna premenna

 GLuint a;         // pomocna premenna
 GLfloat xx,yy,zz; // pomocne premenne

 // nasledovne premenne su pouzivane pri vypoctoch normal
 GLfloat v11,v12,v13;          // 1. vektor
 GLfloat v21,v22,v23;          // 2. vektor
 GLfloat n1,n2,n3,nn1,nn2,nn3; // pomocne premenne

 // vypocet normal stran pre vypocet kolizie
 void CalcCollisionNormals(void);

 // identifikator poslednej textury trojuholnika
 GLuint last_tex_id;

 // koren Solid Leaf Node BSP stromu
 _SolidLeafNode *SLNBSPT;
 
 // chyba pri tvorbe SLNBSP stromu!!!
 GLubyte SLNBSP_error;
 
 // premenne pouzite pri tvorbe SLNBSP stromu
 _TRIANGLE *TriList,*NextTri;
 // premenne pouzite pri deleni trojuholnika na 2, resp. 3 casti
 _TRIANGLE *Split1,*Split2,*Split3;
 
 // tvorba BSP - rekurzivna funkcia!!!
 void BuildSLNBSPT(_SolidLeafNode *node,_TRIANGLE *list);
 // vykreslenie BSP - rekurzivna funkcia!!!
 int DrawSLNBSPT(_SolidLeafNode *node);
 // zrusenie BSP - rekurzivna funkcia!!!
 void KillSLNBSPT(_SolidLeafNode *node);

 // vyber trojuholnika, ktory bude predstavovat deliacu rovinu
 _TRIANGLE *SelectSplitter(_TRIANGLE *list);
 
 // urcenie polohy trojuholnika voci rovine
 int ClassifyTriangle(_TRIANGLE *plane,_TRIANGLE *triangle);

 // rozdelenie trojuholnika na 2, resp. 3 casti
 int SplitTriangle(_TRIANGLE *plane,
                   _TRIANGLE *tri,
                   _TRIANGLE *split1,_TRIANGLE *split2,_TRIANGLE *split3);
				 
 // sposob akym sa vykresli SLNBSP strom
 GLubyte draw_type;
};

#endif
