// *************************************************************************
// * autor:   Marek Sopko                                                  *
// * kontakt: ... Trencin, Slovakia                                        *
// *          http:   www.avatar.host.sk                                   *
// *          e-mail: avatar&host.sk                                       *
// * popis aplikacie: OpenGL Solid Leaf Node BSP Tree                      *
// *************************************************************************

// hlavickove subory *******************************************************
// systemove funkcie
#include <stdlib.h>
// GL a GLU kniznica (OpenGL)
#include <GL/gl.h>
//#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// vlastne funkcie pre pracu s OpenGL - hlavny interface aplikacie
#include "mOGL.h"

// .h subor
#include "priklad19.h"

/***************************************************************************
 * deklaracie a definicie premennych a konstant                            *
 ***************************************************************************/
// sirka, vyska a farebna hlbka (pocet bitov na 1 pixel) SDL okna
#define SCR_WIDTH  720
#define SCR_HEIGHT 486
#define SCR_BPP     32
// pozn.: v X11, VidMode nemeni rozlisenie obrazovky, vo Win32,
// ChangeDisplaySettings moze zmenit BPP

// ukoncenie hlavneho cyklu
int isDone=0;
// SDL okno je/nie je aktivne
int isActive=1;
// polovica sirky a vysky okna
int scr_width2,scr_height2;
// meno aplikacie
const char app_title[64]="OpenGL Solid Leaf Node BSP Tree";

// SDL potrebne premenne
const SDL_VideoInfo *videoInfo=NULL; // informacie o obrazovke
int videoFlags=0;                    // znacky pre SDL_SetVideoMode
SDL_Event event;                     // obsluha udalosti
SDL_Surface *surface;                // SDL kresliaca plocha
Uint8 *keystate;                     // pole SDL stlacanych klaves

// pomocne premenne
int a=0,b=0,c=0;
// pocas startu aplikacie sa nebude brat do uvahy poloha mysi
int go_mouse=0;

// instancia triedy mOGL - hlavny interface aplikacie
mOGL *m_OGL=NULL;

// cislo pouzitej OpenGL kamery
unsigned int Cnum=0;

/***************************************************************************
 * definicie funkcii aplikacie                                             *
 ***************************************************************************/
// uvolnenie alokovanych zdrojov a obnovenie povodnych nastaveni obrazovky *
void APP_quit(int returnCode)
{
 if (m_OGL!=NULL) delete m_OGL; // zrusenie instancie triedy mOGL
 m_OGL=NULL;                    // nastavenie m_OGL=NULL

 SDL_Quit();                    // zrusenie SDL okna

 exit(returnCode);              // ukoncenie aplikacie
}

// inicializacia SDL video subsystemu **************************************
int SDL_initialize(void)
{
 // inicializacia SDL video subsystemu
 if (SDL_Init(SDL_INIT_VIDEO)<0) {
  fprintf(stderr,"Chyba inicializacie SDL video subsystemu: %s\n",SDL_GetError());
  APP_quit(1);
 }

 // ziskanie informacii o SDL video subsysteme
 videoInfo=SDL_GetVideoInfo();
 if (!videoInfo) {
  fprintf(stderr,"Chyba ziskania informacii o SDL video subsysteme: %s\n",SDL_GetError());
  APP_quit(1);
 }

 // nastavenie atributov SDL kresliacej plochy
 // 8 bitov pre cervenu, zelenu a modru farbu
 // 8 bitov pre alfa kanal
 // 16 bitovy zasobnik hlbky
 // 1 pre pouzitie dvojiteho zasobnika pre prekreslovanie
 // 1 bitovy stencil zasobnik
 SDL_GL_SetAttribute(SDL_GL_RED_SIZE,    8);
 SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE,  8);
 SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,   8);
 //SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE,  8);
 SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,  16);
 SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);
 //SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE,1);
 // pozn.: SDL_DOUBLEBUFFER sa pouziva v SDL_GL_SetAttribute,
 // v SDL_SetVideoMode to ovplivni iba 2D prekreslovanie

 // znacky pre SDL_SetVideoMode
 videoFlags =SDL_OPENGL;          // zapne OpenGL v SDL
 videoFlags|=SDL_GL_DOUBLEBUFFER; // alokuje dvojity zasobnik pre prekreslovanie
 videoFlags|=SDL_HWPALETTE;       // ulozi paletu do HW
 videoFlags|=SDL_RESIZABLE;       // umozni menit velkost SDL okna

 // kontrola ci mozno SDL kresliacu plochu umiestnit do HW
 if (videoInfo->hw_available) videoFlags|=SDL_HWSURFACE;
 else videoFlags|=SDL_SWSURFACE;

 // kontrola, ci mozno pracovat s HW blit
 if (videoInfo->blit_hw) videoFlags|=SDL_HWACCEL;

 // zapnutie FSAA ... DOROBIT! zistenie glext
 //SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS,1);
 //SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES,2);

 // nastavenie SDL kresliacej plochy
 surface=SDL_SetVideoMode(SCR_WIDTH,SCR_HEIGHT,SCR_BPP,videoFlags);
 if (!surface) {
  fprintf(stderr,"Chyba nastavenia SDL kresliacej plochy: %s\n",SDL_GetError());
  APP_quit(1);
 }

 // nastavenie nazvu okna a ikony
 SDL_WM_SetCaption(app_title,app_title);
 // schovanie kurzoru mysi
 SDL_ShowCursor(SDL_DISABLE);

 return(0);
}

// modifikacia nastaveni kresliacej plochy po zmene velkosti okna **********
int WND_resize(int width,int height)
{
 // zmena SDL kresliacej plochy (okna)
 surface=SDL_SetVideoMode(width,height,SCR_BPP,videoFlags);
 if (!surface) {
  fprintf(stderr,"Chyba nastavenia SDL kresliacej plochy po zmene velkosti okna: %s\n",SDL_GetError());
  APP_quit(1);
 }

 // INFO: doslo k zmene velkosti okna
 //fprintf(stderr,"ok\n");
 
 // reinicializacia nastaveni kamery po zmene velkosti okna
 if (m_OGL!=NULL) m_OGL->ogl_Camera[Cnum].cam_reinit((GLfloat) width,(GLfloat) height);

 return(0);
}

// obhospodarenie vstupu z mysi ********************************************
void APP_mouse(int button,int state,int x,int y)
{
 // stlacenie laveho a praveho tlacidla mysi
 m_OGL->ogl_Camera[Cnum].isFire1=SDL_GetMouseState(NULL,NULL)&SDL_BUTTON(SDL_BUTTON_LEFT);
 m_OGL->ogl_Camera[Cnum].isFire2=SDL_GetMouseState(NULL,NULL)&SDL_BUTTON(SDL_BUTTON_RIGHT);

 // go_mouse=1 az po prvom nastaveni kurzoru na stred okna,
 // predijde sa tak nechcenej rotacii kamery po starte aplikacie
 x=(x-scr_width2)*go_mouse; y=(y-scr_height2)*go_mouse;

 // rotacia kamery na zakladne pohybu mysi
 m_OGL->ogl_Camera[Cnum].cam_rotate_xy((GLfloat) y,(GLfloat) x);
}

// obhospodarenie vstupu z klavesnice **************************************
void APP_keyboard(void)
{
 keystate=SDL_GetKeyState(NULL); // nacitanie stlacenych klaves

 if (keystate[SDLK_ESCAPE]) isDone=1;                  // ESC

 m_OGL->ogl_Camera[Cnum].goForward=(GLfloat) keystate[SDLK_UP];    // sipka hore
 m_OGL->ogl_Camera[Cnum].goBack=   (GLfloat) keystate[SDLK_DOWN];  // sipka dole
 m_OGL->ogl_Camera[Cnum].goRight=  (GLfloat) keystate[SDLK_RIGHT]; // sipka doprava
 m_OGL->ogl_Camera[Cnum].goLeft=   (GLfloat) keystate[SDLK_LEFT];  // sipka dolava
 m_OGL->ogl_Camera[Cnum].goUp=     (GLfloat) keystate[SDLK_a];     // posun kamery hore
 m_OGL->ogl_Camera[Cnum].goDown=   (GLfloat) keystate[SDLK_z];     // posun kamery dole
 // skok
 m_OGL->ogl_Camera[Cnum].goJump=keystate[SDLK_j];
 // chodza alebo beh
 m_OGL->ogl_Camera[Cnum].goFast=(keystate[SDLK_RSHIFT]||keystate[SDLK_LSHIFT]);

 if (keystate[SDLK_q]) {      // rotacia kamery vlavo okolo doprednej osi
  m_OGL->ogl_Camera[Cnum].cam_rotate_z(-1.0f);
 }
 else if (keystate[SDLK_w]) { // rotacia kamery vpravo okolo doprednej osi
	   m_OGL->ogl_Camera[Cnum].cam_rotate_z(1.0f);
      }

 if (keystate[SDLK_r]) m_OGL->ogl_Camera[Cnum].cam_reset();      // reset pohladu kamery
 if (keystate[SDLK_c]) m_OGL->ogl_Camera[Cnum].CollisionOnOff(); // zap/vyp detekcie kolizii

 if (keystate[SDLK_m]) m_OGL->change_PolygonMode();   // zmena sposobu zobrazenia objektov
 if (keystate[SDLK_l]) m_OGL->change_LightingOnOff(); // zap/vyp osvetlenia
 if (keystate[SDLK_n]) m_OGL->change_NormalsOnOff();  // zap/vyp vykreslovania normal
 if (keystate[SDLK_g]) m_OGL->change_FogOnOff();      // zap/vyp hmly
 if (keystate[SDLK_f]) m_OGL->change_FogMode();       // zmena parametrov hmly
 if (keystate[SDLK_t]) m_OGL->change_TexOnOff();      // zap/vyp texturovania
 if (keystate[SDLK_h]) m_OGL->change_HelpOnOff();     // zap/vyp vypisu helpu
 if (keystate[SDLK_p]) m_OGL->change_PauseMotion();   // zap/vyp casovo zavisleho pohybu objektov
 if (keystate[SDLK_SPACE]) m_OGL->change_DrawType();  // zmena sposobu vykreslovania SLNBSP stromu

 // posun kamery na zaklade vstupu z klavesnice
 m_OGL->ogl_Camera[Cnum].cam_move();
}

// funkcia pre kreslenie OpenGL sceny **************************************
int OGL_SDL_draw_scene(void)
{
 // vykreslenie OpenGL sceny
 m_OGL->gl_draw();

 // vymena zobrazovacich zasobnikov, zobrazenie sceny
 SDL_GL_SwapBuffers();

 return(0);
}

// hlavna vykonavacia funkcia aplikacie ************************************
int main(int argc,char **argv)
{
 scr_width2=SCR_WIDTH/2;   // vypocet polovice sirky okna
 scr_height2=SCR_HEIGHT/2; // vypocet polovice vysky okna

 m_OGL=NULL; // tvorba instancie triedy COGL
 if (m_OGL==NULL) {
  m_OGL=new mOGL();
  if (m_OGL==NULL) {
   fprintf(stderr,"Chyba tvorby instancie triedy mOGL!\n");
   APP_quit(1);
  }
 }

 // inicializacia SDL
 SDL_initialize();
 // inicializacia OpenGL 0=OK, 1=ERROR
 if (m_OGL->gl_initialize()) APP_quit(1);

 WND_resize(SCR_WIDTH,SCR_HEIGHT); // upravenie velkosti SDL okna

 // reset kamery z dovodu spravneho nastartovania casovaca
 m_OGL->ogl_Camera[Cnum].cam_resettime();
 
 // hlavny programovy cyklus
 while (!isDone) {
  // ukladanie a spracovavanie udalosti
  while (SDL_PollEvent(&event)) {
   switch (event.type) {
    case SDL_ACTIVEEVENT:    // kreslenie iba ak je okno aktivne
     if (event.active.gain==0) isActive=0;
     else { isActive=1; m_OGL->ogl_Camera[Cnum].cam_resettime(); }
     break;
    case SDL_VIDEORESIZE:    // zmena velkosti okna
     WND_resize(event.resize.w,event.resize.h);
     break;
    case SDL_QUIT: isDone=1; // zrusenie SDL okna
     break;
	case SDL_MOUSEMOTION:
	 // obsluha mysi
	 APP_mouse(event.button.button,event.button.state,event.button.x,event.button.y);
 	 break;
    default: break;
   }
   // obsluha klavesnice
   APP_keyboard();
   // update parametrov kamery
   m_OGL->ogl_Camera[Cnum].cam_update();
   // casovo zavisla uprava objektov, resp. sceny
   m_OGL->gl_iddle();
  }

  // umiestnenie kurzoru mysi na stred okna
  SDL_WarpMouse(SCR_WIDTH/2,SCR_HEIGHT/2);

  // od tohto okamihu moze byt kamera rotovana na zaklade polohy mysi
  go_mouse=1;

  // kreslenie sceny
  if (isActive) OGL_SDL_draw_scene();
 }

 // ukoncenie aplikacie
 APP_quit(0);

 return(0);
}
